# 异常与中断
---

### xv6 在 qemu 上的时钟中断处理

---

xv6 在 qemu 上运行的时候，它的内核直接被放在地址 0x80000000 处，qemu 直接加载它的内核代码。
我们知道 xv6 内核启动的时候首先会进入 entry.S（一个汇编文件），然后运行其中的第一条汇编指令，然后会跳转到 start.c（一个 C 文件）里面运行，在这个文件里面的 timerinit() 函数就是在 M 态下配置初始化 xv6 时钟中断，然后进入到 S 态运行内核代码。
我们做移植工作的时候不会很关心 xv6 在 qemu 里面是怎么实现时钟中断的，我们只需要知道一点：xv6 是在 M 态对时钟中断进行配置和初始化。

---

### 设计思路
1. 增加中断描述符表 (IDT) ，用于处理不同类型的中断和异常。
2. 增加I/O 高级可编程中断控制器 (I/O APIC) 的初始化和管理代码，用于在多处理器系统中配置和路由中断。
3. 以下是中断处理过程的简要说明：
- **跳转至中断服务例程**
CPU根据中断向量表（IDT）中的条目，跳转到对应的中断服务例程。
- **系统调用中断**：如果中断类型是系统调用（`T_SYSCALL`），则执行以下步骤：
  - 检查当前进程是否被标记为已杀死，如果是，则退出进程。
  - 保存中断帧（`trapframe`）到当前进程的 `tf` 字段。
  - 调用 `syscall` 函数处理系统调用。
  - 再次检查进程是否被杀死，如果是，则退出。
- **硬件中断**：如果中断类型是硬件中断（例如时钟中断 `T_IRQ0 + IRQ_TIMER`），则执行以下步骤：
  - 检查是否是CPU 0，如果是，则更新系统时钟（`ticks`）并唤醒等待时钟的进程。
  - 调用 `lapiceoi` 函数发送EOI（End of Interrupt）信号给Local APIC，表示中断处理完成。
---




#### 中断处理:
- **根据`IDT`调用响应中断服务**
    - 在中断发生时，处理器通过 IDT 调用相应的中断服务例程（ISR）。这些 ISR 通常在其他文件中定义，并在 `trap.c `中通过` IDT` 进行注册。
    - `lapicstartap `函数用于启动额外的处理器，它通过向目标 APIC ID 发送初始化和启动 IPI 来执行此操作。
    - 在发送初始化 IPI 后，它通过发送启动 IPI 来让其他 CPU 执行给定的地址代码。
```cpp
void
cmostime(struct rtcdate *r)
{
	struct rtcdate t1, t2;
	int sb, bcd;

	sb = cmos_read(CMOS_STATB);

	bcd = (sb & (1 << 2)) == 0;

	// make sure CMOS doesn't modify time while we read it
	for(;;) {
		fill_rtcdate(&t1);
		if(cmos_read(CMOS_STATA) & CMOS_UIP)
			continue;
		fill_rtcdate(&t2);
		if(memcmp(&t1, &t2, sizeof(t1)) == 0)
			break;
	}

	// convert
	if(bcd) {
#define    CONV(x)     (t1.x = ((t1.x >> 4) * 10) + (t1.x & 0xf))
		CONV(second);
		CONV(minute);
		CONV(hour  );
		CONV(day   );
		CONV(month );
		CONV(year  );
#undef     CONV
	}

	*r = t1;
	r->year += 2000;
}

```
这段代码主要关注于设置 `Local APIC `并使其能够处理中断，而实际的中断服务例程（ISR）和异常处理会在其他文件中定义，并通过 IDT 进行管理。通过这种方式，xv6 能够在多处理器系统中有效地处理中断。

- **根据中断向量号（`trapno`）来决定应该执行哪个中断处理函数。**

该段代码主要确保操作系统能够响应**硬件事件**，如定时器滴答、硬盘操作、键盘输入和串口通信。每个中断处理函数都负责执行与特定硬件事件相关的任务。处理完中断后，使用`lapiceoi`函数来通知`APIC`中断已经处理完毕，这是现代x86架构中处理中断的标准做法。
```cpp
switch(tf->trapno){
	case T_IRQ0 + IRQ_TIMER:
		if(cpuid() == 0){
			acquire(&tickslock);
			ticks++;
			wakeup(&ticks);
			release(&tickslock);
		}
		lapiceoi();
		break;
	case T_IRQ0 + IRQ_IDE:
		ideintr();
		lapiceoi();
		break;
	case T_IRQ0 + IRQ_IDE+1:
		// Bochs generates spurious IDE1 interrupts.
		break;
	case T_IRQ0 + IRQ_KBD:
		kbdintr();
		lapiceoi();
		break;
	case T_IRQ0 + IRQ_COM1:
		uartintr();
		lapiceoi();
		break;
	case T_IRQ0 + 7:
	case T_IRQ0 + IRQ_SPURIOUS:
		cprintf("cpu%d: spurious interrupt at %x:%x\n",
			cpuid(), tf->cs, tf->eip);
		lapiceoi();
		break
```
#### 恢复执行
中断服务例程完成后，CPU会恢复之前保存的处理器状态，并继续执行被中断的指令。

