# 文件系统 
文件系统的种类众多，而操作系统希望对用户提供一个统一的接口，于是在用户层与文件系统层引入了中间层，这个中间层就称为虚拟文件系统（Virtual File System，VFS）。

VFS 定义了一组所有文件系统都支持的数据结构和标准接口，这样程序员不需要了解文件系统的工作原理，只需要了解 VFS 提供的统一接口即可。

在 Linux 文件系统中，用户空间、系统调用、虚拟机文件系统、缓存、文件系统以及存储之间的关系如下图：
![文件系统结构](https://pic3.zhimg.com/v2-b92499b17ed11bc279689ecdb6efb4b6_1440w.jpg)

---

#### 设计亮点
- 文件系统使用日志机制（通过 log_write）来保证原子性和一致性，能够在系统崩溃时恢复未完成的操作。这在实际操作中能够减少数据丢失的风险。

- 通过对 inode 的缓存管理（inode cache），文件系统能够减少磁盘的 I/O 操作，提高性能。在 inode 访问频繁的情况下，可以避免每次都去磁盘读取 inode 数据。同时每个 inode 都有独立的锁（sleeplock），确保了并发访问时不会发生冲突或死锁
- 对于块的分配，采用了延迟分配的策略，只有在访问时才分配块，从而减少不必要的磁盘操作。
- 目录项是通过 dirlookup 和 dirlink 函数来管理的，能够高效地查找和添加文件到目录中。目录内容是通过固定大小的结构体（dirent）来存储，确保了文件系统的高效性
---
### 文件模块实现
- 我们完成的文件系统由主要是两个模块组成。
#### fs.c
- 块分配与释放 (balloc 和 bfree)
  balloc 函数用于分配一个新的磁盘块。它扫描磁盘块位图（bitmap）以寻找一个空闲块，然后将该块标记为已分配，并返回其块号。
	```cpp	
	for (b = 0; b < sb.size; b += BPB) {
			bp = bread(dev, BBLOCK(b, sb));  // 读取位图块
			for (bi = 0; bi < BPB && b + bi < sb.size; bi++) {
				m = 1 << (bi % 8);  // 计算位图中的一个位
				if ((bp->data[bi/8] & m) == 0) {  // 如果该块未被分配
					bp->data[bi/8] |= m;  // 标记该块为已分配
					log_write(bp);  // 写回日志
					brelse(bp);
					bzero(dev, b + bi);  // 将块清零
					return b + bi;
				}
			}
			brelse(bp);
		}
	```
	bfree 函数用于释放指定的磁盘块。它修改磁盘块位图，标记该块为空闲块。
	```cpp	
	p = bread(dev, BBLOCK(b, sb));  // 读取位图块
    bi = b % BPB;  // 获取位图中的位置
    m = 1 << (bi % 8);
    if ((bp->data[bi/8] & m) == 0)
        panic("freeing free block");  // 释放一个已经空闲的块，出错
    bp->data[bi/8] &= ~m;  // 标记该块为空闲
    log_write(bp);  // 写回日志
    brelse(bp);
	```
- 文件与 inode 管理
  分配 inode (ialloc)：分配一个新的 inode，并初始化其类型。每个 inode 对应一个文件或目录。
	读取 inode (iget)：根据 inode 的编号加载 inode，如果 inode 已经被缓存则直接返回，若未缓存则从磁盘读取。
	更新 inode (iupdate)：将内存中的 inode 数据同步到磁盘，以保证数据的一致性。
- 目录管理
  dirlookup 用于在目录中查找文件，dirlink 用于向目录中添加新的文件项。目录管理通过 inode 来实现文件的映射，目录本质上是包含指向其他 inode 的条目。
  ```cpp	
  for(off = 0; off < dp->size; off += sizeof(de)) {
        if(readi(dp, (char*)&de, off, sizeof(de)) != sizeof(de))  // 读取目录项
            panic("dirlookup read");
        if(de.inum == 0)
            continue;
        if(namecmp(name, de.name) == 0) {  // 找到匹配的文件名
            if(poff)
                *poff = off;
            inum = de.inum;
            return iget(dp->dev, inum);
        }
    }
	```
  namei 和 nameiparent 用于解析文件路径，分别返回文件和父目录的 inode。skipelem 用于从路径中提取每个元素，支持分隔符的跳过。

- 路径解析
  namei 和 nameiparent 用于解析文件路径，分别返回文件和父目录的 inode。skipelem 用于从路径中提取每个元素，支持分隔符的跳过
---
#### file.c
这部分是涉及操作系统中的文件描述符管理,通常用于文件系统中对文件的管理和操作。定义了一些函数来分配、复制、关闭、读取、写入文件，以及获取文件元数据。
- 文件的读取与写入

我们通过fileread 和 filewrite 设置了统一的文件读取和写入接口，支持不同类型的文件，对于管道文件（FD_PIPE），通过 piperead 和 pipewrite 来实现。对于磁盘文件（FD_INODE），通过读取和写入磁盘中的 inode 来实现。
```cpp
if(f->type == FD_PIPE)  // 如果是管道文件，调用 piperead 读取数据
        return piperead(f->pipe, addr, n);
    if(f->type == FD_INODE){  // 如果是磁盘文件，使用 readi 函数读取数据
        ilock(f->ip);  // 锁定 inode，保证数据一致性
        if((r = readi(f->ip, addr, f->off, n)) > 0)  // 从 inode 中读取数据
            f->off += r;  // 更新文件偏移量
        iunlock(f->ip);  // 解锁 inode
        return r;  // 返回实际读取的字节数
    }
```
在写入文件时（filewrite），程序将数据分成多个块进行写入，以避免超出最大日志事务大小的限制。每次写入的数据块大小是通过 MAXOPBLOCKS 和其它相关常量控制的，这样可以有效避免因超大写入而导致的问题。
```cpp
if(f->type == FD_PIPE)  // 如果是管道文件，调用 pipewrite 写入数据
        return pipewrite(f->pipe, addr, n);
    if(f->type == FD_INODE){  // 如果是磁盘文件，使用 writei 函数写入数据
        int max = ((MAXOPBLOCKS-1-1-2) / 2) * 512;  // 计算每次写入的最大数据块大小
        int i = 0;
        while(i < n){
            int n1 = n - i;  // 剩余的数据量
            if(n1 > max)  // 每次写入的最大数据块大小
                n1 = max;

            begin_op();  // 开始操作，确保事务的原子性
            ilock(f->ip);  // 锁定 inode
            if ((r = writei(f->ip, addr + i, f->off, n1)) > 0)  // 写入数据
                f->off += r;  // 更新文件偏移量
            iunlock(f->ip);  // 解锁 inode
            end_op();  // 结束操作，提交事务

            if(r < 0)  // 如果写入失败，返回错误
                break;
            if(r != n1)  // 如果写入的字节数与预期不一致，抛出异常
                panic("short filewrite");
            i += r;  // 更新已经写入的字节数
        }
        return i == n ? n : -1;  // 返回写入的字节数，如果写入的字节数小于预期，返回错误
    }
```
- 不同设备的扩展性

程序通过 devsw（设备表）为不同设备类型提供支持。devsw 表定义了设备的操作接口（如 read 和 write），使得文件系统能够处理不同类型的设备（如磁盘、管道、控制台等）。这增强了文件操作的扩展性和灵活性。
```cpp
int devread(int dev, struct inode *ip, char *buf, int n)
{
    if (dev < 0 || dev >= NDEV) {
        return -1;  // 无效的设备号
    }
    struct devsw *devswp = &devsw[dev];
    if (devswp->read == 0) {
        return -1;  // 没有读取函数
    }
    return devswp->read(ip, buf, n);  // 调用设备的 read 函数
}
```
抽象和设备接口：对于文件类型，系统不仅支持常见的磁盘文件，还支持管道（FD_PIPE），这使得操作系统能够处理多种文件形式，并通过设备表接口来统一管理

---
- #### 相较于原xv6改了什么
- 文件系统实现的代码逻辑与 x86 文件系统非常相似同样包含了五个文件系统层级。

- x86 代码通常会直接处理底层硬件，如磁盘驱动、CPU 指令集等。我们实现了文件系统的核心功能，但不涉及与硬件的直接交互。它更专注于内核内的文件管理逻辑，并依赖于模拟的设备接口（如 bread 和 log_write 等）。它更关注文件系统的高层结构，如磁盘块的分配、inode 管理、目录操作等。
- x86 文件系统实现需要处理多个设备控制器（如 IDE、SATA）、驱动程序、硬件接口等。我们的文件系统的设备接口是抽象的。它依赖于函数如 bread（读取数据块）和 log_write（写入日志），这些函数是通过特定的硬件驱动来实现。
- x86文件系统通常会直接调用硬件驱动来处理磁盘块的读取和写入操作。文件操作依赖于特定的硬件和驱动程序（例如，IDE 驱动、磁盘接口协议等）。而我们设计的文件系统的实现更加抽象化，提供了一个设备抽象层（dev）来处理设备相关的操作。实际的设备驱动代码不在该文件中，而是可能通过如 devsw 数组等方式来管理和调用不同类型设备的操作。
- 我们设计磁盘块的分配和管理通过内核内的逻辑来实现（如 balloc 和 bfree），磁盘操作通过对缓冲区（buf）的管理来进行，而不直接涉及硬件的具体操作。这使得代码更加独立于特定硬件。
---