# **系统调用**
该模块实现了一套进程间共享内存的机制，通过sys_share_mem系统调用允许进程注册共享内存，并通过sys_get_shared系统调用使子进程能够访问这些共享内存。这两个调用确保了内存共享的安全性和有效性，同时通过错误处理和限制措施防止了资源滥用。
**系统共享调用过程如图：**
![系统共享调用过程](https://i-blog.csdnimg.cn/direct/9869dd03327f4471b243f3c5a4de080a.png)

---
### **1.`share_mem`**
该系统调用允许父进程注册和初始化共享内存结构。每个注册的共享内存结构需要一个唯一的名称（限制为20个字符），一个指向起始地址的指针，以及共享内存的大小。每个进程最多可以注册10个共享内存结构，以限制资源使用。注册时会检查是否存在重复名称、参数有效性，以及是否超过允许的注册数。有关错误的处理包括返回指定错误码，例如：同名的共享内存存在、参数不正确或达到共享结构的限制。
```cpp		
	if(argstr(0, &name) < 0 || argptr(1, (void*)&vamem, sizeof(int)) || argint(2, &sz) < 0)		
		return -1;

	for(i=0; ;++i){		
		if(i >= MAXSHARED)
			return -3;
		if(curproc->shared[i].sz == 0)
			break;	
		if(strncmp(name, curproc->shared[i].name, SHAREDNAME) == 0){
			cprintf("fratelo obrisi strukture kada zavrsis sa programom (share_mem)\n");			
			return -2;
		}	
	}
	
	strncpy(shrd.name, name, SHAREDNAME);
	shrd.memstart = vamem;
	shrd.sz = sz; 
	curproc->shared[i] = shrd;
```
---

### **2.`syscal`的执行**

在usertrap函数中进行一些后续操作后，可看到以下代码片段：
```cpp
struct proc *p = myproc();
p->trapframe->epc = r_sepc();
if (r_scause() == 8) {
    if (p->killed)
        exit(-1);
    p->trapframe->epc += 4;
    intr_on();
    syscall();
}
```
该程序精心设计了一套用于进程间通信的共享内存机制。通过sys_share_mem系统调用，进程能够注册并创建共享内存区域，而sys_get_shared系统调用则允许子进程轻松地访问这些预先分配的共享资源。这一机制通过严格的错误处理和资源管理策略，确保了内存共享的安全性和稳定性，同时避免了潜在的资源滥用问题。
```cpp
void syscall(void)
{
    int num;
    struct proc *p = myproc();
    num = p->trapframe->a7;
    if(num > 0 && num < NELEM(syscalls) && syscalls[num]) {
        p->trapframe->a0 = syscalls[num]();
    } else {
        printf("%d %s: unknown sys call %d\n", p->pid, p->name, num);
        p->trapframe->a0 = -1;
    }
}

```

---
### **3.`get_shared`**
此系统调用用于子进程以访问父进程注册的共享内存。通过提供共享结构的名称，子进程可以获得一个指向共享内存的有效指针，借助此指针，子进程可以直接操作共享的内存空间。这需要精确处理以确保只有在确实可以访问的情况下才会设定指针并避免非法的内存访问。
```cpp
	if(argstr(0, &name) < 0 || argptr(1, &addr, sizeof(int)) < 0)	
		return -1;

	if(strncmp("primecom", curproc->name, SHAREDNAME) == 0)
		primecom = 1;	
	if(strncmp("primecalc", curproc->name, SHAREDNAME) == 0 && primecom)
		sleep(curproc, &curproc->parent);

	for(int i=0 ; ; ++i){
		
		if(i >= MAXSHARED)
			break;
	
		if(strncmp(name, curproc->shared[i].name, SHAREDNAME) == 0){

			*addr = curproc->shared[i].memstart;
			if(strncmp("primecom", curproc->name, SHAREDNAME && primecom) == 0)
				wakeup(&curproc->parent);		
			primecom = 0;		

			return 0;}}

	if(strncmp("primecom", curproc->name, SHAREDNAME) == 0 && primecom)
		wakeup(&curproc->parent);
	primecom = 0;
		
	return -2;
```
---
### **4.`getppid`**
对于该系统调用，由于进程中已经有父进程的信息，只需直接返回父进程pid即可
```cpp
return myproc()->parent->pid;
```
---

### **5.`shmget`**
系统调用接口`shmget` 用于创建一个新的共享内存段，或者获取一个已经存在的共享内存段的标识符。

shmflg参数是`shmget()`函数中的一个标志参数，用于指定创建共享内存段时的权限和行为。这个参数的值是通过按位或操作将多个标志组合而成的。

`shmflg`标志参数：
  + `IPC_CREAT`:共享内存不存在就创建(创建后把标识符写到共享内存属性中)，存在就获取与返回共享内存标识符
+ I`PC_EXCL`:不单独使用，一般与`IPC_CREAT`一起使用(`IPC_EXCL` | `IPC CREAT`):如果不存在就创建，如果存在就出错返回;主要用于确保创键的是一个新的共享内存;
---

### **6.返回用户模式**
系统功能调用完成后，从`syscall`函数返回到`usertrap`函数，该函数进行其他操作后，调用`usertrapret`（也定义在kernel/trap.c中）。

这一函数则为返回U模式做准备，包括设置S模式部分CSRs寄存器、在进程`TRAPFRAME`中保存进程在S模式下所需寄存器的值、重设`epc`寄存器等，
最后，需要调用kernel/trampoline.S中的另一个函数——`userret`，其需要两个参数，一个是`TRAPFRAME`在用户空间的虚拟地址（由宏定义），
另一个是用户进程的页表在内核中的地址（已经格式化为正确形式）。

在跳转到`userret`函数后，首先将用户页表还原至页表寄存器satp，并刷新TLB。之后就可以使用虚拟地址，
将`TRAPFRAME`中保存的各个寄存器还原至对应寄存器，最后使用sret指令，返回到用户模式，一个`Syscall`到这里就完成了。