# 进程管理
---
### 概述
该模块负责管理操作系统的进程。它提供了进程的创建、初始化、以及进程表的管理等功能。进程表用于跟踪系统中的所有进程，包括它们的ID、状态、以及其他元数据。

---
 
 ### 数据结构
 该程序所涉及的数据结构如下
 - struct proc: 表示一个进程的结构体，包含进程的状态、PID、内存布局、文件描述符等信息。
- ptable: 全局进程表，用于存储系统中所有进程的proc结构体数组。
- struct spinlock: 自旋锁，用于保护进程表的并发访问。


**代码设计**
 ```cpp
truct {
	struct spinlock lock;
	struct proc proc[NPROC];
} ptable;

static struct proc *initproc;

struct cpu*
mycpu(void)
{
	int apicid, i;

	if(readeflags()&FL_IF)
		panic("mycpu called with interrupts enabled\n");

	apicid = lapicid();
	// APIC IDs are not guaranteed to be contiguous. Maybe we should have
	// a reverse map, or reserve a register to store &cpus[i].
	for (i = 0; i < ncpu; ++i) {
		if (cpus[i].apicid == apicid)
			return &cpus[i];
	}
	panic("unknown apicid\n");
}

struct proc*
myproc(void)
{
	struct cpu *c;
	struct proc *p;
	pushcli();
	c = mycpu();
	p = c->proc;
	popcli();
	return p;
}

 ```
   
---
### 进程初始化
1. **系统启动**:
   - 系统启动时，首先进入引导加载程序，然后加载内核。
2. **进程表初始化**:
   - 在内核初始化阶段，会调用`pinit`函数来初始化进程表（ptable）。
   ```c
   void pinit(void) {
       initlock(&ptable.lock, "ptable");
   }
   ```
   - `initlock`函数初始化一个自旋锁，用于保护进程表，防止多处理器系统中的并发访问问题。
   
---
### 进程创建
进程创建通常涉及以下几个步骤：
1. **分配进程控制块（PCB）**:
   在提供的代码中，每个进程由`proc`结构体表示，这是进程控制块（PCB）的一种形式。进程创建的第一步是从进程表中分配一个新的`proc`结构。
2. **初始化进程属性**:
   分配了`proc`结构后，需要初始化它的属性，如进程ID、状态、程序计数器、寄存器状态等。
3. **分配资源**:
   为新进程分配必要的资源，如内存空间、文件描述符等。
4. **设置进程状态**:
   将新进程的状态设置为就绪（Ready），这样它就可以被调度器调度执行。
5. **将进程添加到进程表**:
   将新创建的进程添加到进程表`ptable`中，以便操作系统可以跟踪和管理它。

```c
struct proc* allocproc(void) {
    struct proc *p;
    // 分配一个进程控制块
    p = &ptable.proc[nextpid++];
    // 初始化进程属性
    p->state = UNUSED;
    p->pid = nextpid;
    // ... 其他初始化代码 ...
    // 分配资源
    // ... 资源分配代码 ...
    // 设置进程状态为就绪
    p->state = RUNNABLE;
    // 将进程添加到进程表
    // ... 添加到进程表的代码 ...
    return p;
}
```
---


### 进程执行
7. **执行进程**:
   - 进程获得CPU后，开始执行其指令。这涉及到设置程序计数器（PC）到正确的位置，并开始执行。
8. **使用自旋锁**:
   - 在执行过程中，如果进程需要访问共享资源（如全局变量、文件描述符等），它必须首先获取相应的自旋锁。
   ```c
   acquire(&ptable.lock);
   // 访问共享资源...
   release(&ptable.lock);
   ```
   - 获取自旋锁会阻塞其他试图访问相同资源的进程，直到锁被释放。
  
---

### 进程切换
9. **中断和上下文切换**:
   - 在执行过程中，可能会发生中断（如时钟中断、I/O中断等），这会导致当前进程被挂起，操作系统进行上下文切换。
   - 上下文切换涉及保存当前进程的状态（如寄存器值）并加载下一个要运行进程的状态。
### 进程结束
  
---
10. **进程终止**:
    - 当进程完成其任务或被终止时，系统会回收分配给该进程的资源，并从进程表中移除其PCB。
以下是进程运行过程的简化流程图：
```
系统启动
    |
初始化进程表（pinit）
    |
创建新进程（allocproc）
    |
设置进程状态为就绪
    |
调度器选择进程运行
    |
获取CPU资源
    |
执行进程
    |----> 访问共享资源
    |         |----> 获取自旋锁
    |         |----> 访问资源
    |         |----> 释放自旋锁
    |
发生中断
    |
上下文切换
    |
进程继续执行或被挂起
    |
进程完成任务或被终止
    |
回收资源并从进程表中移除
```

