### 编写中断异常代码时遇到的问题

##### 问题1：如果一个中断在另一个中断处理过程中发生，并且它们访问共享资源，可能会导致数据竞态。


**解决方法**：使用原子操作或禁用中断来保护共享资源。
```c
void interrupt_handler1(struct trapframe *tf) {
    disable_interrupts();
    shared_var++;
    enable_interrupts();
}
void interrupt_handler2(struct trapframe *tf) {
    disable_interrupts();
    shared_var--;
    enable_interrupts();
}
```

---

##### 问题2：错误的中断优先级设置
我们在编写代码的过程中，曾错误的中断优先级设置，导致系统运行出错，经过小组讨论后，我们发现由于我们设置中断优先级时，错误地将网络中断的优先级时低于时钟中断从而导致关键的中断被低优先级的中断阻塞。

**解决方法**：重新评估并设置正确的中断优先级。
```c
set_interrupt_priority(NETWORK_INTERRUPT, 1); // 更高优先级
set_interrupt_priority(TIMER_INTERRUPT, 2);
```

---

##### 问题3：中断服务例程中的错误处理不足
**困难**：我们开始编写代码是忽略了ISR中没有适当的错误处理，导致了系统在遇到硬件故障时崩溃。

**解决方法**：在ISR中添加错误检查和恢复逻辑。
```cpp
void disk_interrupt_handler(struct trapframe *tf) {
    if (!read_disk_data()) {
        handle_disk_error();
    }
}
```

---

### 编写中断异常代码时遇到的问题

**1. 文件系统操作的原子性被破坏**
**问题**：如果文件系统操作（如写文件）在执行过程中被中断，并且中断处理程序也涉及到文件系统的操作，那么原始操作可能只完成了一部分，导致数据不一致。


**解决方法**：确保文件系统操作是原子的，或者在操作期间禁用中断。

---

**2. 中断延迟导致文件系统性能下降**
**问题**：如果中断处理程序执行时间过长，可能会导致文件系统请求的响应时间变慢。

**解决方法**：优化中断处理程序，减少执行时间，或者将部分工作推迟到非中断上下文中执行。

---

**3. 缓存不一致性**
**问题**：如果中断处理程序修改了文件系统缓存，而缓存更新没有及时同步到磁盘，可能会出现缓存不一致。

```c
void file_flush(struct file *f) {
    // 假设这里没有确保缓存一致性
    write_disk(f->f_cache);
}
```
**解决方法**：确保在缓存被修改后，及时将更改写入磁盘，或者在系统崩溃后能够恢复缓存的一致性。

---
### 其他问题
**问题**：在编写完系统调用进行测试时，出现了奇怪的错误，用shell程序不断的运行测试用例，会出现奇怪的错误，且错误不一定，有时是指令非法错误，有时是访存权限不够，并且当快速运行用例，比慢速运行用例出现的错误概率要大，经过不断的试错，初次判断时用户栈太小的缘故，可能时栈满却依然有进栈操作，导致访存权限不够，但关于快慢运行程序的问题无法解释，但用quem模拟时不会出现该错误，这就非常奇怪，但在增大用户栈之后似乎错误解决了就暂时放弃更改，该来的终究会来，最后代码汇总完用自动运行代码进行总体的测试时，又出现了类似的错误，通过不断的查找资料，并结合quem模拟和真实板子不同点，发生这种情况应该是TBL相关问题，并且每次问题都是出现在会更改页表的系统调用之后，所以应该页表更改后是TBL没有及时刷新，导致出现奇怪错误，刷新TBL的代码为sfence.vma，且每次执行系统调用xv6都会在执行完系统调用进行刷新，这就很奇怪，明明已经刷新了为什么还是出现错误，
**解决方案**：经过不断查找资料终于找到解决方案就是在sfence.vma前后加上fence和fence.i。